///////////////////////////////////////////////////////////////////////////
//  This program computes a test data set for Garcia's examples
//  of the FFT on pages 153 to 158.  Here, the data consist of
//  N data point pairs {t,y(t)} constructed from:
//       y(t) = sin(2*pi*f*t + phi)
//  where t = j*tau and tau = 1 with j=0..N-1.
//
//  This is a modified version of ftdemo.cpp from Garcia's
//  "Numerical Methods for Physics" web site.  It also uses
//  Garcia's fft.cpp routine.
//
//              thomas l. gibson  March 18, 2006
///////////////////////////////////////////////////////////////////////////
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <cmath>
#include <iomanip>
using namespace std;

         
int main() 
{
  //* Initialize the sine wave time series to be transformed.
  cout << "Enter the number of points: ";
  int N; cin >> N;
  cout << "Enter frequency of the sine wave: ";
  double freq; cin >> freq;
  cout << "Enter phase of the sine wave: ";
  double phase; cin >> phase;
  double tau = 1;  // Time increment
  const double pi = M_PI;
  double t,y;
  register int i;
  ofstream out_1;
  out_1.open("fft_ex_data.in", ios::out);
  out_1 << "Sine-wave FFT example data: " << N << " points with "
        << "frequency = " << freq << " and phase = " << phase << endl;
  out_1 << "time     signal\n";
  for( i=0; i<N; i++ ) 
  {
    t = i * tau;                       // t = [0, tau, 2*tau, ... ]            
    y = sin(2*pi*t*freq + phase);      // Sine wave time series
    out_1 <<  setiosflags(ios::fixed)  << setprecision(5) << t 
          << "\t" 
          <<  setprecision(8) << y 
          << "\n";
  }
  cout << "data file --> fft_ex_data.in\n";
  out_1.close();
  return EXIT_SUCCESS;
}  
