#!/usr/bin/perl -w
##########################################################################
# This perl script is designed to pick a file using "zenity" for
# FFT processing assuming a data set {t,y) that has
# two or more title lines followed by N (time , amplitude) value pairs,
# one pair per line separated by a single tab.  This file will strip off 
# the carriage character, write the titles to an input file and the 
# (time, amplitude) values to a separate data file.  It will also determine 
# the largest  number of points np2, for which np2 = 2^m with m integer 
# (required for Garcia's FFT algorithm).
# np2 will be written to a separate data file for processing by fft_tlg.cc
#
#      thomas l. gibson March 27, 2012
#
##########################################################################
$title_file = "fft_titles_tlg.in";
$data_file = "fft_data_tlg.in";
$npoints_file = "fft_npoints_tlg.in";

$rawfile=`zenity --file-selection --title="Select a File"`;
if ($rawfile ne "")
{
  print ("File --> $rawfile selected.\n");
}
else
{
 print ("No file selected.\n");
 exit (0);
}

open (IN, "<$rawfile") || die "Couldn't open $rawfile \n";

if (-e $title_file)
{
  unlink $title_file;
}

if (-e $data_file)
{
  unlink $data_file;
}

open (OUT_t, ">$title_file") || die "Couldn't open $title_file \n";
open (OUT_d, ">$data_file") || die "Couldn't open $data_file \n";

$n_data = 0;
while( defined ( $line = <IN>) )
{
  chomp($line);
  $line =~ s/\r//g;                  # remove carriage returns
  if ($line =~ /^(\d+\.\d+\t-*\d+\.\d+)/)
  {
    print OUT_d ("$line \n");
    $n_data++;
  }
  else
  {
    print OUT_t ("$line \n");
  }
}
print ("Number of data values in file = $n_data \n");
if ($n_data < 2)
{
  print ("Dude, your data set is seriously screwed up!\n");
  print ("\tCheck the web page for input data specifications.\n");
  exit;
}

$m = int (log($n_data) / log(2) + 0.5);
$n_test = int ( 2**$m + 0.5);
if ($n_test > $n_data)
{
  $m--;
}
  $np2 = int( 2**$m );

print ("Number of values (2^n) to be used for FFT = $np2 \n");

open (OUT_npoints, ">$npoints_file") || die "Couldn't open file $npoints_file \n";
print OUT_npoints ("$np2 \n");

close (IN);
close (OUT_t);
close (OUT_d);
close (OUT_npoints);

system("./fft_tlg");

####if (-e "powSpec.par")
####{
####  system("xmgrace -param powSpec.par powSpec.plt");
####}
####else
####{
####  system("xmgrace powSpec.plt");
####}
system("gnuplot powSpec.gp");
exit (0);
