///////////////////////////////////////////////////////////////////////////
//  This program will compute the direct sum Fourier transform of a data set
//  of time, amplitude values {ti,Ai}.
//
//  This is a modified version of ftdemo.cpp from Garcia's
//  "Numerical Methods for Physics" web site.  It also uses
//  Garcia's fft.cpp routine.
//
//              thomas l. gibson  March 29, 2012
///////////////////////////////////////////////////////////////////////////
#include <ctime>
using namespace std;

#include "NumMeth.h"

int main() 
{
  clock_t start, end;
  double elapsed;



  const double pi = M_PI;
//* Initialize the sine wave time series to be transformed.
  cout << "Enter the number of points: ";
  int N; cin >> N;
  cout << "Enter frequency of the sine wave: ";
  double freq; cin >> freq;
  cout << "Enter phase of the sine wave: ";
  double phase; cin >> phase;
  double tau = 1;  // Time increment
  const double arg1=-2.0*M_PI/N;  // Constant part of FT argument
  Matrix t(N+1), y(N+1), f(N+1);
  register int i,j,k;  
  for (i=1; i<=N+1; i++)
  {
    t(i) = (i - 1)*tau;
    y(i) = sin(2*M_PI*freq*t(i) + phase);
    f(i) = (i - 1)/(N * tau);
  }
// Compute the transform using Direct Sum Fourier transform algorithm.
  start = clock();
  Matrix ytReal(N+1), ytImag(N+1);
  for (k=0; k<N+1; k++)
  {
    ytReal(k+1) = 0.0;
    ytImag(k+1) = 0.0;
    for (j=0; j<N+1; j++)
    {
      ytReal(k+1) += y(j+1)*cos(arg1*j*k);
      ytImag(k+1) += y(j+1)*sin(arg1*j*k);
    }
  }
// Compute the power spectrum
  Matrix powSpec(N+1);
  for( k=1; k<=N+1; k++ )
  {
    powSpec(k) = ytReal(k)*ytReal(k) + ytImag(k)*ytImag(k);
  }
// Print out the plotting variables: 
//    y, f, ytReal, ytImag, powspec
  ofstream yOut("y.plt"), fOut("f.plt"), 
	   ytRealOut("ytReal.plt"), ytImagOut("ytImag.plt"), 
	   powSpecOut("powSpec.plt");
  for( i=1; i<=N+1; i++ ) 
  {
    yOut << t(i) << " " << y(i) << endl;
    fOut << f(i) << endl;
  }
  for( i=1; i<=N+1; i++ )  // No DC Suppression -- (include f_0 component) 
  {
    ytRealOut << f(i) << " " << ytReal(i) << endl;
    ytImagOut << f(i) << " " << ytImag(i) << endl;
    powSpecOut << f(i) << " " << powSpec(i) << endl;
  }
  cout << "Fourier Transform Data: "<< endl;
  yOut.close();
  fOut.close();
  ytRealOut.close();
  ytImagOut.close();
  powSpecOut.close();

  cout << "\tNumber of points used for direct sum FT: " << N << endl;
  cout << "\tTime increment tau = " << tau << " (s)" << endl;
  cout << "\n\n" << endl;
//  cout << "Plot files: DC Component (f_0) omitted" << endl;
  cout << "\tData file --> y.plt" << endl;
  cout << "\tFrequency file --> f.plt" << endl;
  cout << "\tReal FFT file --> ytReal.plt" << endl;
  cout << "\tImag FFT file --> ytImag.plt" << endl;
  cout << "\tPower Spectrum file --> powSpec.plt" << endl;
  end = clock();
  elapsed = ((double) (end - start)) / CLOCKS_PER_SEC;
  cout << "\tElapsed CPU time = " << elapsed << " seconds" << endl;
  return 0;
}
