///////////////////////////////////////////////////////////////////////////
//  This program will compute the discrete Fourier transform of a data set
//  of time, amplitude values {ti,Ai}.
//
//  This is a modified version of ftdemo.cpp from Garcia's
//  "Numerical Methods for Physics" web site.  It also uses
//  Garcia's fft.cpp routine.
//
//              thomas l. gibson  January 13, 2005
//
//  Revised by  thomas l. gibson  April 02, 2012
///////////////////////////////////////////////////////////////////////////
#include <cmath>
using namespace std;

#include "NumMeth.h"

void fft( Matrix& RealA, Matrix& ImagA);
         
int main() 
{
  clock_t start,end;
  double elapsed;
  const double pi = M_PI;
  int N;
  double tau, T;
  char title[76];

  ifstream rd_npoints;
  rd_npoints.open("fft_npoints_tlg.in", ios::in);
  ifstream rd_title;
  rd_title.open("fft_titles_tlg.in", ios::in);
  ifstream rd_data;
  rd_data.open("fft_data_tlg.in", ios::in);
  rd_npoints >> N;
  rd_npoints.close();

  Matrix t(N), y(N), f(N);
  register int i,j,k;  
  for (i=1; i<=N; i++)
  {
    rd_data >> t(i) >> y(i);
  }
  rd_data.close();
  tau = t(2) - t(1);
  cout << "t(2) = " << t(2) << "\tt(1) = " << t(1) << endl; //tlg-test

  cout << "tau = " << tau << endl;  //tlg-test
  T = t(N) - t(1);
  cout << "T = " << T << endl; //tlg-test
  for (i=1; i<=N; i++)
  {
    f(i) = (i-1)/(N*tau);       //tlg-test: original (i-1)/T
  }  
  cout << "f(1) = " << f(1) << "\tf(N) = " << f(N) << endl; //tlg-test  
// Compute the transform using fast Fourier transform (FFT) algorithm.
  start = clock();
  Matrix ytReal(N), ytImag(N);
  ytReal = y;
  ytImag.set(0.0);     // Copy data for input to fft
  fft(ytReal, ytImag);

  // Compute the power spectrum
  Matrix powSpec(N);
  for( k=1; k<=N; k++ )
  {
    powSpec(k) = ytReal(k)*ytReal(k) + ytImag(k)*ytImag(k);
  }
// Print out the plotting variables: 
//    y, f, ytReal, ytImag, powspec
  ofstream yOut("y.plt"), fOut("f.plt"), 
	   ytRealOut("ytReal.plt"), ytImagOut("ytImag.plt"), 
	   powSpecOut("powSpec.plt");
  for( i=1; i<=N; i++ ) 
  {
    yOut << t(i) << " " << y(i) << endl;
    fOut << f(i) << endl;
  }
  for( i=2; i<=N; i++ )  // DC Suppression -- (omit f_0 component) 
  {
    ytRealOut << f(i) << " " << ytReal(i) << endl;
    ytImagOut << f(i) << " " << ytImag(i) << endl;
    powSpecOut << f(i) << " " << powSpec(i) << endl;
  }
  cout << "For data file: "<< endl;
  while ( rd_title.getline(title, 76) )
  {
    cout << "\t" << title << endl;
  }
  rd_title.close();
  yOut.close();
  fOut.close();
  ytRealOut.close();
  ytImagOut.close();
  powSpecOut.close();

  cout << "\tNumber of points used for discrete FFT: " << N << endl;
  cout << "\tTime increment tau = " << tau << " (s)" << endl;
  cout << "\tTotal time interval T = " << T << " (s)" << endl; 
  cout << "\n\n" << endl;
  cout << "Plot files: DC Component (f_0) omitted" << endl;
  cout << "\tData file --> y.plt" << endl;
  cout << "\tFrequency file --> f.plt" << endl;
  cout << "\tReal FFT file --> ytReal.plt" << endl;
  cout << "\tImag FFT file --> ytImag.plt" << endl;
  cout << "\tPower Spectrum file --> powSpec.plt" << endl;
  end = clock();
  elapsed = ((double) (end - start)) / CLOCKS_PER_SEC;
  cout << "\tElapsed CPU time = " << elapsed << " seconds" << endl;  
  return 0;
}
